@@ -1,6 +1,6 @@
 iht <- function(...) {
   ldots <- list(...)
-  if ("model" %in% names(ldots) | is.character(ldots[[1]][1])) {
+  if ("model" %in% names(ldots) || is.character(ldots[[1]][1])) {
     conTestD(...)
   } else {
     conTest(...)
