@@ -1,5 +1,5 @@
 print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"), 
-                            color = TRUE, ...) {
+                            hypo_rate_threshold = 1, color = TRUE, ...) {
 
   # Ensure the object is of class 'benchmark_means'
   if (!inherits(x, "benchmark")) {
@@ -10,6 +10,14 @@ print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"),
   
   ldots <- list(...)
   
+  # compute hypothesis rate
+  x$hypothesis_rate <- lapply(x$combined_values$rgw_combined, 
+                              calculate_hypothesis_rate, q = hypo_rate_threshold)
+  
+  # no effect names
+  NE_names <- names(x$hypothesis_rate) %in% c("pop_est = No-effect", "pop_es = 0")
+  x$hypothesis_rate[NE_names] <- as.numeric(NA)
+  
   # number of failed bootstrap runs
   empty_lists_count <- sapply(x$combined_values$gw_combined, function(x) { 
     attr(x, "empty_lists_count") } )
@@ -67,20 +75,21 @@ print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"),
       reset <- "}"
     }
   } else {
+    # R Markdown cannot handle color codes (console, html and latex)
     blue <- green <- orange <- reset <- ""
   }
   
   text_gw  <- paste0("Benchmark: Percentiles of ", orange, goric_type, " Weights", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
   text_rgw <- paste0("Benchmark: Percentiles of ", orange, "Ratio-of-", goric_type, "-weights", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
-  text_rlw <- paste0("Benchmark: Percentiles of ", orange, "Ratio-of-likelihood-weights", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
-  text_ld  <- paste0("Benchmark: Percentiles of ", orange, "Differences in Likelihood Values", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
+  text_rlw <- paste0("Benchmark: Percentiles of ", orange, "Ratio-of-log-likelihood-weights", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
+  text_ld  <- paste0("Benchmark: Percentiles of ", orange, "Differences in Log-likelihood Values", blue, " for the Preferred Hypothesis '", pref_hypo, "'")
   
   cat("\n")
   #cat(strrep("=", 70), "\n")
   cat(paste0(blue, "Benchmark Results", reset), "\n")
   cat(strrep("-", 70), "\n")
   cat(sprintf("Preferred Hypothesis: %s%s%s\n", green, pref_hypo, reset))
-  cat(sprintf("Error probability Preferred Hypothesis vs. complement: %s%s%s\n", green, error_prob_pref_hypo, reset))
+  cat(sprintf("Error probability Preferred Hypothesis vs. Complement: %s%s%s\n", green, error_prob_pref_hypo, reset))
   if (inherits(x, "benchmark_means")) {
     cat(sprintf("Number of Groups: %s%s%s\n", green, ngroups, reset))
   } else {
@@ -113,9 +122,7 @@ print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"),
                       "it is advisable to increase the number of bootstrap iterations.")
     message("---\n", text_msg)
   }
-# -------------------------------------------------------------------------
 
-  
   # Normalize output_type to lowercase
   output_type <- tolower(output_type)
   
@@ -125,7 +132,8 @@ print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"),
       text_gw,
       function() {
         for (pop_es in names(x$benchmarks_goric_weights)) {
-          print_rounded_es_value(x$benchmarks_goric_weights[[pop_es]], pop_es, model_type, green, reset)
+          print_rounded_es_value(x$benchmarks_goric_weights[[pop_es]], pop_es, 
+                                 model_type, green, reset)
         }
       }, nchar(text_gw), text_color = blue, reset = reset
     )
@@ -133,37 +141,55 @@ print.benchmark <- function(x, output_type = c("rgw", "gw", "rlw", "ld", "all"),
   
   # Print benchmarks for ratio Goric-weights percentiles
   if ("all" %in% output_type || "rgw" %in% output_type) {
+    
+    # Loop over alle sets in both benchmarks_ratio_goric_weights and hypothesis_rate
+    for (pop_es_name in names(x$benchmarks_ratio_goric_weights)) {
+      x$benchmarks_ratio_goric_weights[[pop_es_name]] <- cbind(
+        x$benchmarks_ratio_goric_weights[[pop_es_name]],
+        hypothesis_rate = x$hypothesis_rate[[pop_es_name]] # x$hypothesis_rate
+      )
+    }
+    
+    if (any(NE_names)) {
+      x$benchmarks_ratio_goric_weights[NE_names][[1]] <- 
+        x$benchmarks_ratio_goric_weights[NE_names][[1]][, -7, drop = FALSE]
+    }
+    
     print_section(
       text_rgw,
       function() {
         for (pop_es in names(x$benchmarks_ratio_goric_weights)) {
-          print_rounded_es_value(x$benchmarks_ratio_goric_weights[[pop_es]], pop_es, model_type, green, reset)
+          print_rounded_es_value(x$benchmarks_ratio_goric_weights[[pop_es]], pop_es, 
+                                 model_type, green, reset)
         }
       }, nchar(text_rgw), text_color = blue, reset = reset
     )
   }
   
-  # Print benchmarks for ratio likelihood-weights percentiles
+  # Print benchmarks for ratio log-likelihood-weights percentiles
   if ("all" %in% output_type || "rlw" %in% output_type) {
     print_section(
       text_rlw,
       function() {
         for (pop_es in names(x$benchmarks_ratio_ll_weights)) {
-          print_rounded_es_value(x$benchmarks_ratio_ll_weights[[pop_es]], pop_es, model_type, green, reset)
+          print_rounded_es_value(x$benchmarks_ratio_ll_weights[[pop_es]], pop_es, 
+                                 model_type, green, reset)
         }
       }, nchar(text_rlw), text_color = blue, reset = reset
     )
   }
   
-  # Print benchmarks for difference likelihood-values percentiles
+  # Print benchmarks for difference log-likelihood-values percentiles
   if ("all" %in% output_type || "ld" %in% output_type) {
     print_section(
       text_ld,
       function() {
         for (pop_es in names(x$benchmarks_difLL)) {
-          print_rounded_es_value(x$benchmarks_difLL[[pop_es]], pop_es, model_type, green, reset)
+          print_rounded_es_value(x$benchmarks_difLL[[pop_es]], pop_es, model_type, 
+                                 green, reset)
         }
       }, nchar(text_ld), text_color = blue, reset = reset
     )
   }
+  return(invisible(x))
 }
