@@ -3,6 +3,8 @@
 # 2. LL + PT
 # 3. IC values (AIC, ORIC, GORIC, GORICA)
 # 4. IC weights or (Bayesian) posterior model probs. 
+# 5. Output from gorica function
+# 6. Output from escalc (metafor package)
 
 # In case of an equal-evidence approach, aggregating evidence from, say, 5 studies 
 # with n=100 observations is the same as obtaining evidence from 1 study 
@@ -10,34 +12,12 @@
 # In the added-evidence approach, the aggregated evidence from, says, 5 studies 
 # is stronger than as if the data were combined (as if that was possible).
 
-# TODO
-
-#0. benchmark functie
-
-#1. geef een opmerking als bijvoorbeeld de loglik niet beschikbaar zijn, zoals bij gorica values en weights
-
-#2. meerdere fit objecten
-
-#3. geef meer informatieve foutmelding als bv. est = data ipv vector
-
-#4. bereken de overlap tussen hypothesen en test het tov. het complement
-
-#5. multiple hyposthesen vs. complement
-
-#6. add type = "average" and update manual (DONE) 
-
-#7. Hoe groot mag het verschil zijn tussen gelijke log-likelihood value
-
-#8. Hard programeren interpretatie output. Bij exact gelijk check voor overlap
-# Bij 3 means controlen wanneer bij == de loglik nagenoeg gelijk is met het complement
-# kijk naar sample esimates en gebruik dat voor toekomstig onderzoek.
-
-#evSyn <- function(object, ...) { UseMethod("evSyn") }
-
-evSyn_est       <- function(object, ...) UseMethod("evSyn_est")
-evSyn_LL        <- function(object, ...) UseMethod("evSyn_LL")
-evSyn_ICweights <- function(object, ...) UseMethod("evSyn_ICweights")
-evSyn_ICvalues  <- function(object, ...) UseMethod("evSyn_ICvalues")
+# evSyn_est       <- function(object, ...) UseMethod("evSyn_est")
+# evSyn_LL        <- function(object, ...) UseMethod("evSyn_LL")
+# evSyn_ICweights <- function(object, ...) UseMethod("evSyn_ICweights")
+# evSyn_ICvalues  <- function(object, ...) UseMethod("evSyn_ICvalues")
+# evSyn_ICratios  <- function(object, ...) UseMethod("evSyn_ICratios")
+# evSyn_escalc    <- function(object, ...) UseMethod("evSyn_escalc")
 # -------------------------------------------------------------------------
 ## est (list + vec) + cov (list + mat)
 ## LL (list + vec) + PT (list + vec)
@@ -48,103 +28,11 @@ evSyn_ICvalues  <- function(object, ...) UseMethod("evSyn_ICvalues")
 ## object = LL, PT = PT
 ## object = IC weights + rowsum check
 ## object = IC values
+## object = Ratio IC weights
+
 # -------------------------------------------------------------------------
 
-
-# evSyn <- function(object, ...) {
-#   
-#   arguments <- list(...)
-#   
-#   checkArguments <- function(args) {
-#     pnames <- c("VCOV", "PT", "hypotheses", "type", "comparison", "hypo_names")
-#     pm <- pmatch(names(args), pnames, nomatch = 0L)
-#     if (any(pm == 0L)) { 
-#       stop("Restriktor Error: ", names(args[which(pm == 0L)]), " invalid argument(s).")
-#     }
-#   }
-#   
-#   if (length(arguments)) checkArguments(arguments)
-# 
-#   # if all vectors sum to 1, object contains IC weights
-#   obj_isICweights <- FALSE
-#   VCOV <- arguments$VCOV
-#   PT   <- arguments$PT
-#   
-#   # input is a list of gorica objects and everthing is inherited and passsed to 
-#   # the evSyn_LL() function.
-#   # isGoric <- sapply(object, function(x) inherits(x, "con_goric"))
-#   isGoric <- vapply(object, function(x) inherits(x, "con_goric"), logical(1))
-#   
-#   
-#   
-#   
-#   if (!all(isGoric)) {
-#     # object must be a list
-#     if (!is.list(object)) {
-#       stop("Restriktor ERROR: object must be a list of numeric vectors.", call. = FALSE)
-#     }
-#     # object must be a list with numeric vectors
-#     obj_isnum <- sapply(object, is.numeric)
-#     if ( !any(obj_isnum) ) {
-#       stop("Restriktor ERROR: object must be a list of numeric vectors.", call. = FALSE)
-#     }
-#     
-#     if (!is.null(VCOV) && !is.list(VCOV)) {
-#       stop("Restriktor ERROR: VCOV must be a list of matrices.", call. = FALSE)
-#     } else if (!is.null(VCOV) ) {
-#       # check if it is a list of matrices
-#       VCOV_isMat <- sapply(VCOV, is.matrix)
-#       if (!any(VCOV_isMat)) {
-#         stop("Restriktor ERROR: VCOV must be a list of matrices.", call. = FALSE)  
-#       }
-#     }
-#   
-#     if (!is.null(PT) && !is.list(PT)) {
-#       stop("Restriktor ERROR: PT must be a list of numeric vectors.", call. = FALSE)
-#     } else if ( !is.null(PT) ) {
-#       # check if it is a list of numeric vectors
-#       PT_isNum <- sapply(PT, is.numeric)
-#       if (!any(PT_isNum)) {
-#         stop("Restriktor ERROR: PT must be a list of numeric vectors.", call. = FALSE)  
-#       }
-#     }
-#   
-#     if (!is.null(VCOV) & !is.null(PT)) {
-#       stop("Restriktor ERROR: both VCOV and PT are found, which confuses me.")
-#     }
-#     
-#     if ( all(abs(sapply(object, sum) - 1) <= sqrt(.Machine$double.eps)) ) {
-#      obj_isICweights <- TRUE  
-#     } 
-#   }
-#   
-#   ## est (vector) + VCOV (matrix)
-#   ## check if list contains a vector/matrix with the the parameter estimates
-#   if (all(isGoric)) {
-#     evSyn_gorica.list(object, ...)
-#   } else if (!is.null(VCOV)) {
-#     evSyn_est(object, ...)
-#   } else if (!is.null(PT)) {
-#     # LL + PT
-#     evSyn_LL(object, ...)
-#   } else if (is.null(VCOV) & is.null(PT) & obj_isICweights) { 
-#     # IC weights
-#     if (!is.null(arguments$type) && arguments$type == "equal") {
-#       message("The added-evidence method is the only available approach when weights are included in the input.")
-#     }
-#     evSyn_ICweights(object, ...)
-#   } else if (is.null(VCOV) & is.null(PT) & !obj_isICweights) { 
-#     # IC values
-#     if (!is.null(arguments$type) && arguments$type == "equal") { 
-#       message("The added-evidence method is the only available approach when the input consists of GORIC(A) values.")
-#     }
-#     evSyn_ICvalues(object, ...)
-#   } else {
-#     stop("Restriktor Error: I don't know how to handle the input.")
-#   }
-# }
-
-evSyn <- function(object, ...) {
+evSyn <- function(object, input_type = NULL, ...) {
   
   arguments <- list(...)
   
@@ -153,10 +41,36 @@ evSyn <- function(object, ...) {
   
   isGoric <- vapply(object, function(x) inherits(x, "con_goric"), logical(1))
   
-  if (all(isGoric)) {
-    return(evSyn_gorica.list(object, ...))
+  # Handle input_type explicitly if provided
+  if (!is.null(input_type)) {
+    input_type <- tolower(input_type)
+    if (input_type == "est_vcov") {
+      return(evSyn_est(object, ...))
+    } else if (input_type == "ll_pt") {
+      return(evSyn_LL(object, ...))
+    } else if (input_type == "icweights") {
+      return(evSyn_ICweights(object, ...))
+    } else if (input_type == "icratios") {
+      return(evSyn_ICratios(object, ...))
+    } else if (input_type == "icvalues") {
+      return(evSyn_ICvalues(object, ...))
+    } else if (input_type == "gorica") {
+      return(evSyn_gorica(object, ...))
+    } else if (input_type == "escalc") {
+      return(evSyn_escalc(object, ...))
+    } else {
+      stop(paste0("Restriktor ERROR: Unknown input_type ", sQuote(input_type), "."))
+    }
   }
   
+  if (all(isGoric)) {
+    return(evSyn_gorica(object, ...))
+  }
+  
+  if (inherits(object, "escalc")) {
+    return(evSyn_escalc(object, ...))
+  } 
+  
   if (!is.list(object) || !any(vapply(object, is.numeric, logical(1)))) {
     stop("Restriktor ERROR: object must be a list of numeric vectors.")
   }
@@ -167,14 +81,17 @@ evSyn <- function(object, ...) {
     }
   }
   
-  checkListContent(VCOV, is.matrix, "VCOV must be a list of matrices.")
-  checkListContent(PT, is.numeric, "PT must be a list of numeric vectors.")
+  checkListContent(lst = VCOV, fun = is.matrix, msg = "VCOV must be a list of matrices.")
+  checkListContent(lst = PT, fun = is.numeric, msg = "PT must be a list of numeric vectors.")
   
   if (!is.null(VCOV) && !is.null(PT)) {
     stop("Restriktor ERROR: both VCOV and PT are found, which confuses me.")
   }
   
+  # if they are weights, the sum of all vectors must be 1.
   obj_isICweights <- all(abs(vapply(object, sum, numeric(1)) - 1) <= sqrt(.Machine$double.eps))
+  # Check if they are IC ratios: each vector should end with 1.
+  obj_isICratios <- all(vapply(object, function(x) tail(x, n = 1) == 1, logical(1)))
   
   if (!is.null(VCOV)) {
     return(evSyn_est(object, ...))
@@ -185,13 +102,17 @@ evSyn <- function(object, ...) {
       message("The added-evidence method is the only available approach when weights are included in the input.")
     }
     return(evSyn_ICweights(object, ...))
+  } else if (obj_isICratios) {
+    if (!is.null(arguments$type) && arguments$type == "equal") {
+      message("The added-evidence method is the only available approach when weights are included in the input.")
+    }
+    return(evSyn_ICratios(object, ...))
   } else {
     if (!is.null(arguments$type) && arguments$type == "equal") {
       message("The added-evidence method is the only available approach when the input consists of GORIC(A) values.")
     }
     return(evSyn_ICvalues(object, ...))
   }
-  
   stop("Restriktor Error: I don't know how to handle the input.")
 }
 
@@ -199,10 +120,10 @@ evSyn <- function(object, ...) {
 # -------------------------------------------------------------------------
 # GORIC(A) evidence synthesis based on the (standardized) parameter estimates and 
 # the covariance matrix
-evSyn_est.list <- function(object, ..., VCOV = list(), hypotheses = list(),
-                           type = c("equal", "added", "average"), 
-                           comparison = c("unconstrained", "complement", "none"),
-                           hypo_names = c()) {
+evSyn_est <- function(object, ..., VCOV = list(), hypotheses = list(),
+                      type = c("added", "equal", "average"), 
+                      comparison = c("unconstrained", "complement", "none"),
+                      hypo_names = c()) {
   
   if (missing(comparison)) 
     comparison <- "unconstrained"
@@ -469,7 +390,7 @@ evSyn_est.list <- function(object, ..., VCOV = list(), hypotheses = list(),
               Final_ratio_LL_weights = Final.ratio.LL.weights
               )
   
-  class(out) <- c("evSyn.est", "evSyn")
+  class(out) <- c("evSyn_est", "evSyn")
   
   return(out)
 }
@@ -477,8 +398,8 @@ evSyn_est.list <- function(object, ..., VCOV = list(), hypotheses = list(),
 
 # -------------------------------------------------------------------------
 # GORIC(A) evidence synthesis based on log likelihood and penalty values
-evSyn_LL.list <- function(object, ..., PT = list(), type = c("equal", "added", "average"),
-                          hypo_names = c()) {
+evSyn_LL <- function(object, ..., PT = list(), type = c("added", "equal", "average"),
+                     hypo_names = c()) {
   
   if (missing(type)) 
     type <- "added"
@@ -611,7 +532,7 @@ evSyn_LL.list <- function(object, ..., PT = list(), type = c("equal", "added", "
               Final_ratio_LL_weights = Final.ratio.LL.weights
               )
   
-  class(out) <- c("evSyn.LL", "evSyn")
+  class(out) <- c("evSyn_LL", "evSyn")
   
   return(out)
 }
@@ -620,7 +541,7 @@ evSyn_LL.list <- function(object, ..., PT = list(), type = c("equal", "added", "
 
 # -------------------------------------------------------------------------
 # GORIC(A) evidence synthesis based on AIC or ORIC or GORIC or GORICA values
-evSyn_ICvalues.list <- function(object, ..., hypo_names = c()) {
+evSyn_ICvalues <- function(object, ..., hypo_names = c()) {
   
   IC <- object
   S  <- length(IC)
@@ -674,7 +595,7 @@ evSyn_ICvalues.list <- function(object, ..., hypo_names = c()) {
               Cumulative_GORICA_weights  = CumulativeGoricaWeights,
               Final_ratio_GORICA_weights = Final.ratio.GORICA.weights)
   
-  class(out) <- c("evSyn.ICvalues", "evSyn")
+  class(out) <- c("evSyn_ICvalues", "evSyn")
   
   return(out)
 }
@@ -682,8 +603,9 @@ evSyn_ICvalues.list <- function(object, ..., hypo_names = c()) {
 
 
 # -------------------------------------------------------------------------
-# GORIC(A) evidence synthesis based on AIC or ORIC or GORIC or GORICA weights or (Bayesian) posterior model probabilities
-evSyn_ICweights.list <- function(object, ..., priorWeights = NULL, hypo_names = c()) {
+# GORIC(A) evidence synthesis based on AIC or ORIC or GORIC or GORICA weights or 
+# (Bayesian) posterior model probabilities
+evSyn_ICweights <- function(object, ..., priorWeights = NULL, hypo_names = c()) {
   
   Weights <- object
   S <- length(Weights)
@@ -726,21 +648,74 @@ evSyn_ICweights.list <- function(object, ..., priorWeights = NULL, hypo_names =
               Cumulative_GORICA_weights  = CumulativeWeights,
               Final_ratio_GORICA_weights = Final.ratio.GORICA.weights)
   
-  class(out) <- c("evSyn.ICweights", "evSyn")
+  class(out) <- c("evSyn_ICweights", "evSyn")
   
   return(out)
+}
+
+
+# -------------------------------------------------------------------------
+# GORIC(A) evidence synthesis based on the ratio of AIC or ORIC or GORIC or GORICA 
+# weights or (Bayesian) posterior model probabilities
+evSyn_ICratios <- function(object, ..., priorWeights = NULL, hypo_names = c()) {
   
+  Weights <- object
+  S <- length(Weights)
+  Weights <- do.call(rbind, Weights)
+  NrHypos <- ncol(Weights)
+  
+  if (is.null(priorWeights)) {
+    priorWeights <- rep(1/(NrHypos), (NrHypos))
+  }
+  # To make it sum to 1 (if it not already did)
+  priorWeights <- priorWeights / sum(priorWeights) 
+  
+  if (is.null(hypo_names)) {
+    hypo_names <- paste0("H", 1:(NrHypos)) 
+  }
+  
+  CumulativeWeights <- matrix(NA, nrow = (S+1), ncol = (NrHypos))
+  colnames(Weights) <- colnames(CumulativeWeights) <- hypo_names
+  
+  sequence <- paste0("Studies 1-", 1:S)
+  sequence[1] <- "Studies 1"
+  
+  rownames(CumulativeWeights) <- c(sequence, "Final")
+  CumulativeWeights[1, ] <- priorWeights * Weights[1, ] / sum( priorWeights * Weights[1, ] )
+  
+  for (s in 2:S) {
+    CumulativeWeights[s, ] <- CumulativeWeights[(s-1), ] * Weights[s, ] / 
+      sum( CumulativeWeights[(s-1), ] * Weights[s, ] )
+  }
+  
+  CumulativeWeights[(S+1), ] <- CumulativeWeights[S, ]
+  
+  Final.weights <- CumulativeWeights[S, ]
+  Final.ratio.GORICA.weights <- Final.weights %*% t(1/Final.weights)
+  rownames(Final.ratio.GORICA.weights) <- hypo_names
+  colnames(Final.ratio.GORICA.weights) <- paste0("vs. ", hypo_names)
+  
+  out <- list(type                       = "added",
+              GORICA_weight_m            = Weights,
+              Cumulative_GORICA_weights  = CumulativeWeights,
+              Final_ratio_GORICA_weights = Final.ratio.GORICA.weights)
+  
+  class(out) <- c("evSyn_ICratios", "evSyn")
+  
+  return(out)
 }
 
 
 
+# -------------------------------------------------------------------------
 # list with goric objects
-evSyn_gorica.list <- function(object, ..., type = c("equal", "added", "average"), 
-                              hypo_names = c()) {
+evSyn_gorica <- function(object, ..., type = c("added", "equal", "average"), 
+                         hypo_names = c()) {
   
   # Check if all objects are of type "con_goric"
   if (!all(vapply(object, function(x) inherits(x, "con_goric"), logical(1)))) {
-    stop("Restriktor ERROR: the object must be a list with fitted objects from the goric() function", call. = FALSE)
+    stop("Restriktor ERROR: the object must be a list with fitted objects from the goric() function", 
+         call. = FALSE)
   }
   
   # Create a list for the evSyn_LL.list function
@@ -752,317 +727,37 @@ evSyn_gorica.list <- function(object, ..., type = c("equal", "added", "average")
   )
   
   # Call the evSyn_LL.list function and return the result
-  result <- do.call(evSyn_LL.list, conList)
+  result <- do.call(evSyn_LL, append(conList, list(...)))
+  class(result) <- c(class(result), "evSyn_gorica")
   
   return(result)
 }
 
 
-print.evSyn <- function(x, digits = max(3, getOption("digits") - 4), ...) {
+evSyn_escalc <- function(data, yi_col = "yi", vi_cols = "vi", 
+                         cluster_col = "trial", outcome_col = NULL, ...) {
   
-  # added or equal approach
-  #type <- x$type
-  
-  cat(sprintf("restriktor (%s): %s Evidence Synthesis results:\n", 
-              packageDescription("restriktor", fields = "Version"), x$type))
-  
-  if (!is.null(x$Cumulative_GORICA_weights)) {
-    cat("\nFinal GORICA weights:\n") 
-    cgw <- sapply(x$Cumulative_GORICA_weights["Final", , drop = FALSE], 
-                  FUN = function(x) format_numeric(x, digits = digits))
-    names(cgw) <- colnames(x$Cumulative_GORICA_weights)
-    print(cgw, print.gap = 2, quote = FALSE, right = TRUE)
-    cat("---\n")
-  }
-  
-  cat("\nRatio final GORICA weights:\n")  
-  formatted_weights <- apply(x$Final_ratio_GORICA_weights, c(1, 2), 
-                             function(val) format_numeric(val, digits = digits))
-  print(formatted_weights, print.gap = 2, quote = FALSE, right = TRUE)
-  cat("\n")
-  
-  message(x$messages$mix_weights)
-}
-
-
-summary.evSyn <- function(object, ...) {
-  x <- object
-  class(x) <- NULL
-  
-  ans <- list(
-    type = x$type,
-    n_studies = nrow(x$GORICA_weight_m[,, drop = FALSE]),
-    hypotheses = x$hypotheses,
-    GORICA_weight_m = x$GORICA_weight_m,
-    GORICA_m = x$GORICA_m,
-    LL_weights_m = x$LL_weights_m,
-    Cumulative_LL_weights = x$Cumulative_LL_weights,
-    Cumulative_LL = x$Cumulative_LL,
-    LL_m = x$LL_m,
-    PT_m = x$PT_m,
-    Cumulative_GORICA_weights = x$Cumulative_GORICA_weights,
-    Cumulative_GORICA = x$Cumulative_GORICA
+  results <- extract_est_vcov_outcomes(
+    data = data,
+    outcome_col = outcome_col,
+    yi_col = yi_col,
+    vi_cols = vi_cols,
+    cluster_col = cluster_col
   )
-
-  if (!is.null(x$PT_m)) {
-    sequence    <- paste0("Studies 1-", 1:ans$n_studies)
-    sequence[1] <- "Studies 1"
-    Cumulative_PT <- apply(x$PT_m[,, drop = FALSE], 2, cumsum)  
-    Cumulative_PT <- matrix(Cumulative_PT, nrow = nrow(x$PT_m), 
-                            dimnames = list(sequence, colnames(x$PT_m)))
-    if (x$type %in% c("equal", "average")) {
-      Cumulative_PT <- Cumulative_PT / seq_len(ans$n_studies)
-    } 
-    ans$Cumulative_PT <- Cumulative_PT
-  }
   
-  final <- c()
+  # Access the parameter estimates and vcov blocks
+  yi_list <- results$yi_list
+  vcov_blocks <- results$vcov_blocks
   
-  if (!is.null(x[["Cumulative_GORICA_weights"]])) {
-    fcgw <- t(x[["Cumulative_GORICA_weights"]]["Final", ])
-    rownames(fcgw) <- "GORICA weights"
-    colnames(fcgw)  <- colnames(x[["Cumulative_GORICA_weights"]])
-    final <- rbind(final, fcgw)
-  }
   
-  if (!is.null(x[["Cumulative_GORICA"]])) {
-    fcgv <- t(x[["Cumulative_GORICA"]]["Final", ])
-    rownames(fcgv) <- "GORICA values"
-    final <- rbind(final, fcgv)
-  }
-
-  if (!is.null(x[["Cumulative_LL_weights"]])) {
-    fcgw <- t(x[["Cumulative_LL_weights"]]["Final", ])
-    rownames(fcgw) <- "Log-likelihood weights"
-    colnames(fcgw)  <- colnames(x[["Cumulative_LL_weights"]])
-    final <- rbind(final, fcgw)
-  }
+  conList <- list(
+    object = yi_list,
+    VCOV = vcov_blocks
+  )
   
-  if (!is.null(x[["Cumulative_LL"]])) {
-    fcllv <- t(x[["Cumulative_LL"]][ans$n_studies, ])
-    rownames(fcllv) <- "Log-likelihood values"
-    final <- rbind(final, fcllv)
-  }
+  # Call the evSyn_LL.list function and return the result
+  result <- do.call(evSyn_est, append(conList, list(...)))
+  class(result) <- c(class(result), "evSyn_escalc")
   
-  if (!is.null(ans$Cumulative_PT)) { 
-    fcptv <- t(ans$Cumulative_PT[ans$n_studies, ])
-    rownames(fcptv) <- "Penalty term values"
-    final <- rbind(final, fcptv)
-  }
-  
-  ans$Final_Cumulative_results <- final
-  
-  ans$Ratio_GORICA_weight_mu <- x$ratio_GORICA_weight_mu
-  ans$Ratio_GORICA_weight_mc <- x$ratio_GORICA_weight_mc
-  ans$Final_ratio_GORICA_weights <- x$Final_ratio_GORICA_weights
-  ans$Final_ratio_LL_weights <- x$Final_ratio_LL_weights
-  
-  ans$messages <- list(mix_weights = x$messages$mix_weights)
-  class(ans) <- "summary.evSyn"
-  
-  ans
+  return(result)
 }
-
-
-
-## print.summary function
-print.summary.evSyn <- function(x, digits = max(3, getOption("digits") - 4), ...) {
-  
-  # added or equal approach
-  type <- x$type
-  # number of studies
-  S <- x$n_studies
-  
-  cat(sprintf("restriktor (%s): ", packageDescription("restriktor", fields = "Version")))
-  cat(paste(type, "Evidence Synthesis results:\n"), sep = "")
-  
-  indentation <- "    "  # Four spaces for indentation
-  
-  cat("\nStudy-specific results:\n")
-  
-  if (!is.null(x$GORICA_weight_m)) {
-    cat("\n    GORICA weights:\n")  
-    formatted_gw <- apply(x$GORICA_weight_m[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_gw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$GORICA_m)) {
-    cat("\n    GORICA values:\n")  
-    formatted_gv <- apply(x$GORICA_m[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_gv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-
-  if (!is.null(x$LL_weights_m)) {
-    cat("\n    Log-likelihood weights:\n")  
-    formatted_llw <- apply(x$LL_weights_m[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_llw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$LL_m)) {
-    cat("\n    Log-likelihood values:\n")  
-    formatted_llv <- apply(x$LL_m[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_llv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$PT_m)) {
-    cat("\n    Penalty term values:\n")  
-    formatted_ptv <- apply(x$PT_m[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_ptv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  cat("\nCumulative results:\n")
-  
-  if (!is.null(x[["Cumulative_GORICA_weights"]])) {
-    cat("\n    GORICA weights:\n")  
-    formatted_cgw <- apply(x$Cumulative_GORICA_weights[1:S, , drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_cgw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x[["Cumulative_GORICA"]])) {
-    cat("\n    GORICA values:\n")  
-    formatted_cgv <- apply(x$Cumulative_GORICA[1:S, , drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_cgv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x[["Cumulative_LL_weights"]])) {
-    cat("\n    Log-likelihood weights:\n")  
-    formatted_cllw <- apply(x$Cumulative_LL_weights[1:S, , drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_cllw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$LL_m)) {
-    cat("\n    Log-likelihood values:\n")  
-    formatted_cllv <- apply(x$Cumulative_LL[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_cllv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$PT_m)) {
-    cat("\n    Penalty term values:\n")  
-    formatted_cptv <- apply(x$Cumulative_PT[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_cptv, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  
-  cat("\nFinal results:\n")
-  formatted_final <- apply(x$Final_Cumulative_results[,,drop = FALSE], c(1,2), function(x) format_numeric(x, digits = digits))
-  captured_output <- capture.output(print(formatted_final, row.names = TRUE, right = TRUE, quote = "FALSE"))
-  adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-  cat(paste0(adjusted_output, "\n"), sep = "")
-  
-  
-  cat("\nFinal ratios:\n")
-  
-  if (!is.null(x$Final_ratio_GORICA_weights)) {
-    cat("\n    GORICA weights:\n")  
-    formatted_frgw <- apply(x$Final_ratio_GORICA_weights, c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_frgw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  if (!is.null(x$Final_ratio_LL_weights)) {
-    cat("\n    Log-likelihood weights:\n")  
-    formatted_frllw <- apply(x$Final_ratio_LL_weights, c(1,2), function(x) format_numeric(x, digits = digits))
-    captured_output <- capture.output(print(formatted_frllw, row.names = TRUE, right = TRUE, quote = "FALSE"))
-    adjusted_output <- gsub("^", indentation, captured_output, perl = TRUE)
-    cat(paste0(adjusted_output, "\n"), sep = "")
-    cat("    ---\n")
-  }
-  
-  cat("\n")
-  message(x$messages$mix_weights)
-}
-
-
-
-## plot function
-plot.evSyn <- function(x, ...) {
-  namesH <- colnames(x$GORICA_weight_m)
-  NrHypos_incl <- ncol(x$GORICA_weight_m[,,drop = FALSE])
-  S <- nrow(x$GORICA_weight_m[,,drop = FALSE])
-  Name_studies <- as.factor(1:S)
-  
-  GORICA_weight_m <- x$GORICA_weight_m
-  CumulativeGoricaWeights <- x$Cumulative_GORICA_weights
-  
-  # Create data frame for per study weights
-  if (all(is.na(GORICA_weight_m))) {
-    per_study_df <- NULL
-    times <- 1
-  } else {
-    per_study_df <- data.frame(study = rep(Name_studies, NrHypos_incl),
-                               weight = c(GORICA_weight_m))
-    per_study_df$weight_type <- "per study"
-    times <- 2
-  }
-  # Create data frame for cumulative weights
-  cumulative_df <- data.frame(study = rep(Name_studies, NrHypos_incl),
-                              weight = c(CumulativeGoricaWeights[1:S, ]))
-  cumulative_df$weight_type <- "cumulative"
-  
-  # Combine data frames
-  plot_data <- rbind(per_study_df, cumulative_df)
-  plot_data$variable <- rep(rep(namesH, each = S), times = times)
-  
-  # Create plot
-  ggplot(plot_data, aes(x = .data[['study']],  
-                        y = .data[['weight']], 
-                        shape    = .data[['weight_type']], 
-                        linetype = .data[['weight_type']], 
-                        color    = .data[['variable']])) +
-    geom_point(size = 3) +
-    geom_line(data = plot_data[plot_data[['weight_type']] == "cumulative", ], 
-              aes(group = .data[['variable']]), linewidth = 1) +
-    scale_color_brewer(palette = "Dark2") +
-    theme(
-      plot.margin = unit(c(1,1,1,1), "cm"),
-      legend.position = "bottom",
-      legend.margin = margin(t = 10, r = 0, b = 3, l = 0),
-      legend.key = element_blank(),
-      legend.text = element_text(size = 12),
-      axis.text.x  = element_text(size = 12), axis.text.y = element_text(size = 12),
-      axis.title.x = element_text(size = 14, vjust = -3), axis.title.y = element_text(size = 14, vjust = 5),
-      panel.background = element_blank(),
-      axis.line = element_line(colour = "black"),
-      plot.title = element_text(hjust = 0.5, size = 14)
-    ) +
-    scale_y_continuous(limits = c(0, 1), n.breaks = 10) +
-    scale_x_discrete(expand = c(0, 0.05)) +
-    labs(x = "Studies", y = "GORIC(A) weights",
-         title = "Cumulative GORIC(A) weights and GORIC(A) weights per study",
-         shape = "", color = "", linetype = "") +
-    guides(shape = guide_legend(override.aes = list(linetype = 0)))
-}
-
-
