@@ -72,12 +72,14 @@ print.summary.restriktor <- function(x, digits = max(3, getOption("digits") - 2)
     cat("\nHeteroskedastic robust standard errors:", se.type ,"\n")
   }
   
-  if (!(inherits(x, c("summary.conGLM", "summary.conMLM"))) && !is.na(x$R2.reduced)) {
-    if (all((x$R2.org - x$R2.reduced) < 1e-08)) {
-     cat("Multiple R-squared remains", sprintf("%5.3f", x$R2.org),"\n")
-    } else {
-     cat("Multiple R-squared reduced from", sprintf("%5.3f", x$R2.org), "to", 
-         sprintf("%5.3f", x$R2.reduced),"\n")  
+  if (!(inherits(x, c("summary.conGLM", "summary.conMLM"))) && !is.null(x$R2.reduced)) {
+    if (!is.na(x$R2.reduced)) {
+      if (all((x$R2.org - x$R2.reduced) < 1e-08)) {
+       cat("Multiple R-squared remains", sprintf("%5.3f", x$R2.org),"\n")
+      } else {
+       cat("Multiple R-squared reduced from", sprintf("%5.3f", x$R2.org), "to", 
+           sprintf("%5.3f", x$R2.reduced),"\n")  
+      }
     }
   }
   
